#version 100
precision highp float;
precision highp int;

varying vec4 v_color;
varying vec2 v_texCoords;
varying float v_type;
varying float v_rand;

uniform sampler2D u_texture0;
uniform sampler2D u_texture1;
uniform vec2 uv_regionoffset;
uniform vec2 uv_regionsize;
uniform float time;

vec4 back_color = vec4(0,0,0,1);
float glow_border = 0.08;
float glow_radius = 0.1;
float glow2_border = 0.2;
float glow2_radius = 0.3;
float core_border = 0.03;
float core_radius = 0.04;
vec2 disc_center = vec2(0.5, 0.5);
float disc_radius = 0.45;
float border_size = 0.002;
float rspeed = 3.0;

// texture atlas stuff
float coreRegularDownscale = 6.3;
vec2 coreRegularOffset = vec2(0.0,0);
float coreBigDownscale = 2.1;
vec2 coreBigOffset = vec2(0.5,0);
vec2 sprites = vec2(2.0,1.0);


float oscillate(float vMin, float vMax, float t) {
    float halfRange = (vMax - vMin) / vMax * 0.5;
    return (sin(t) * halfRange + (1.0 - halfRange)) * vMax;
}

float makeDisc(vec2 p, float b, float r) {
	return smoothstep(r + b, r - b, sqrt(dot(p, p)));
}

float makeCircle(vec2 p, float b, float r) {
    float dist =  sqrt(dot(p, p));
    return  1.0 + smoothstep(r, r+b, dist) - smoothstep(r-b, r, dist);
}

vec2 rot(float r, float deg) {
	return vec2(r * cos(radians(deg)), r * sin(radians(deg)));
}
vec2 rotateUV(vec2 uv, float rotation, vec2 mid)
{
    return vec2(
      cos(rotation) * (uv.x - mid.x) + sin(rotation) * (uv.y - mid.y) + mid.x,
      cos(rotation) * (uv.y - mid.y) - sin(rotation) * (uv.x - mid.x) + mid.y
    );
}
vec4 getPartsRegion(sampler2D samp){
    vec2 texoff = vec2(0.0,0.0);

    if(v_color.a < 0.05)
    {
           texoff = vec2(0.25,0.0);
    }
    else if(v_color.a > 0.05 && v_color.a < 0.1)
    {
           texoff = vec2(0.5,0.0);
    }
    else if(v_color.a > 0.1 && v_color.a < 0.2 )
    {
           texoff = vec2(0.75,0.0);
    }
    else if(v_color.a > 0.2 && v_color.a < 0.3 )
    {
           texoff = vec2(0.0, 0.250);
    }
    else if(v_color.a > 0.3 && v_color.a < 0.4 )
    {
           texoff = vec2(0.250, 0.25);
    }
    else if(v_color.a > 0.4 && v_color.a < 0.5 )
    {
           texoff = vec2(0.5, 0.25);
    }
    else if(v_color.a > 0.5 && v_color.a < 0.6 )
    {
           texoff = vec2(0.75, 0.25);
    }
    else if(v_color.a > 0.6 && v_color.a < 0.7 )
    {
           texoff = vec2(0.0, 0.5);
    }
    else if(v_color.a > 0.7 && v_color.a < 0.8 )
    {
         texoff = vec2(0.250, 0.5);
    }
    else if(v_color.a > 0.8 && v_color.a < 0.9 )
    {
         texoff = vec2(0.25, 0.5);
    }
    else if(v_color.a > 0.9 && v_color.a < 0.95 )
    {
         texoff = vec2(0.5, 0.5);
    }
    else if(v_color.a > 0.95 )
    {
         texoff = vec2(0.75, 0.5);
    }

    vec2 flip = vec2(1.0,1.0);
    vec2 flipoff = vec2(0.0,0.0);

    if(v_rand > 0.5){flip = vec2(1.0,1.0);}
    if(v_rand < 0.5){flip = vec2(-1.0,1.0);flipoff = vec2(-0.75,0.0);}
    vec2 texuv = (v_texCoords * flip) * vec2(0.25, 0.25) + texoff + flipoff;

    return texture2D(samp, texuv);
}

vec4 drawBigFood(float t){
    float rnd = (v_color.a * 2.0) - 1.0;
    float rndt = t * rnd;
    float alpha = t+v_color.a;

    vec4 tex = getPartsRegion(u_texture1);
    vec4 blood = texture2D(u_texture1, v_texCoords * vec2(0.25, 0.25) + vec2(0.0, 0.75) );

    vec2 glowuv1 = v_texCoords * vec2(0.25, 0.25) + vec2(0.75, 0.75) ;
    glowuv1 = rotateUV(glowuv1, rndt * 1.63, vec2(0.75 + 0.125, 0.75 + 0.125));
    vec4 glowtex1 = texture2D(u_texture0, glowuv1);

    vec2 glowuv2 = v_texCoords * vec2(0.25, 0.25) + vec2(0.75, 0.75) ;
    glowuv2 = rotateUV(glowuv2, rndt * 0.43, vec2(0.75 + 0.125, 0.75 + 0.125));
    vec4 glowtex2 = texture2D(u_texture0, glowuv2);
    float mulglow = glowtex1.r * glowtex2.r;

    //vec4 bdisk = mix(vec4(0,0,0,0.0), vec4(1,1,1, 1), makeDisc(glow1pos, 0.001, 0.125));// * oscillate(0.0, 1.0, alpha * 12.5);
    //vec4 ftex = mix(vec4(0,0,0,0.0), tex, makeDisc(glow1pos, 0.001, 0.225));
    return mix(vec4(blood.rgb* 0.76, blood.a * 0.5 ) + vec4(mulglow,mulglow*0.87,mulglow*0.4,mulglow * 0.5)/* + mulglow*/,  tex, tex.a);
}

vec4 drawSmallFood(float t){
    float rnd = (v_color.a * 2.0) - 1.0;
    float rndt = t * rnd;

    vec2 texoff = vec2(0.0,0.0);
    if(v_color.a < 0.1)
    {
           texoff = vec2(0.25,0.0);
    }
    else if(v_color.a > 0.1 && v_color.a < 0.2 )
    {
           texoff = vec2(0.5,0.0);
    }
    else if(v_color.a > 0.2 && v_color.a < 0.3 )
    {
           texoff = vec2(0.75, 0.0);
    }
    else if(v_color.a > 0.3 && v_color.a < 0.4 )
    {
           texoff = vec2(0.0, 0.25);
    }
    else if(v_color.a > 0.4 && v_color.a < 0.5 )
    {
           texoff = vec2(0.25, 0.25);
    }
    else if(v_color.a > 0.5 && v_color.a < 0.6 )
    {
           texoff = vec2(0.5, 0.25);
    }
    else if(v_color.a > 0.6 && v_color.a < 0.7 )
    {
           texoff = vec2(0.75, 0.25);
    }
    else if(v_color.a > 0.7 && v_color.a < 0.8 )
    {
         texoff = vec2(0.0, 0.5);
    }
    else if(v_color.a > 0.8 && v_color.a < 0.9 )
    {
         texoff = vec2(0.5, 0.5);
    }

    vec2 flip = vec2(1.0,1.0);
    vec2 flipoff = vec2(0.0,0.0);
    if(v_rand > 0.5){flip = vec2(1.0,1.0);}
    if(v_rand < 0.5){flip = vec2(-1.0,1.0);flipoff = vec2(-0.75,0.0);}
    vec2 texuv = (v_texCoords * flip) * vec2(0.25, 0.25) + texoff + flipoff;

    // glowing
    vec4 glowtex = vec4(0);
    vec2 diskpos = v_texCoords - disc_center;
    float radius = length(diskpos);
    if(radius < 0.495)
    {
        vec2 glowuv = v_texCoords * 0.25 + vec2(0.75, 0.75);
        glowuv = rotateUV(glowuv, rndt * 0.33, vec2(0.75+0.125,0.75 + 0.125));
        glowtex = texture2D(u_texture0, clamp(glowuv, vec2(0.0,0.0), vec2(1.0,1.0)));
    }

    // two colors blending for glowing
    vec4 glowcol1 = vec4(0.90, 0.57, 0.99, 1.0);
    vec4 glowcol2 = vec4(0.47, 0.86, 0.99, 1.0);
    vec4 glow = mix(glowcol1, glowcol2, oscillate (0.0, 1.0, rndt));

    //texuv = (v_texCoords * vec2(0.25, 0.25)) * vec2(-1.0,1.0) + vec2(-0.75,0.0);

    vec4 tex = texture2D(u_texture0, texuv );// * vec4(v_color.rgb, 1.0);
    float os = oscillate (0.63, 0.7, rndt);
    return mix( vec4(glow.rgb, glowtex.r * os) , tex, tex.a);//coredisk + glow2disk + glow1disk ;
}

vec4 drawToxicFood(float t){
    float rnd = (v_color.a * 2.0) - 1.0;
    float rndt = t * rnd;
    float tt = t * v_color.a;

    // мухоморы
    vec2 flip = vec2(1.0,1.0);
    vec2 texoff = vec2(0.25,0.5);
    if(v_rand > 0.5){flip = vec2(1.0,1.0);texoff = vec2(0.25,0.5);}
    if(v_rand < 0.5){flip = vec2(-1.0,1.0);texoff = vec2(0.5,0.5);}
    vec4 tex0 = texture2D(u_texture0, (v_texCoords * flip) * 0.25 + texoff);

    vec2 glowuv1 = v_texCoords * vec2(0.25, 0.25) + vec2(0.75, 0.75) ;
    glowuv1 = rotateUV(glowuv1, rndt * 1.63, vec2(0.75 + 0.125, 0.75 + 0.125));
    vec4 glowtex1 = texture2D(u_texture0, glowuv1);

    vec2 glowuv2 = v_texCoords * vec2(0.25, 0.25) + vec2(0.75, 0.75) ;
    glowuv2 = rotateUV(glowuv2, rndt * 0.43, vec2(0.75 + 0.125, 0.75 + 0.125));
    vec4 glowtex2 = texture2D(u_texture0, glowuv2);

    float mulglow = glowtex1.r * glowtex2.r;

    return mix( vec4(mulglow,glowtex1.r*0.97,glowtex1.r*0.5,mulglow * 1.0)/* + mulglow*/,  tex0, tex0.a);
}

void main()
{
    if (v_type == 0.0) {
        gl_FragColor = drawBigFood(time);// big food

    } else if (v_type == 1.0) {
        gl_FragColor = drawSmallFood(time);// regular food

    } else if (v_type == 2.0) {
        gl_FragColor = drawToxicFood(time);

    } else {
        discard;//gl_FragColor = vec4(v_texCoords.x, v_texCoords.y,0.0,oscillate(0.3,0.6,new_t));
    }

}